@@ -10,9 +10,8 @@ import zope.interface
 log = logging.getLogger(__name__)
 
 
-class Handler(object):
-
-    zope.interface.implements(gocept.amqprun.interfaces.IHandler)
+@zope.interface.implementer(gocept.amqprun.interfaces.IHandler)
+class Handler:
 
     def __init__(self, queue_name, routing_key, handler_function,
                  arguments=None, principal=None):
@@ -31,6 +30,10 @@ class Handler(object):
     def __call__(self, message):
         return self.handler_function(message) or []
 
+    def __repr__(self):
+        return '<gocept.amqprun.handler.Handler({!r}, {!r}, {!r})>'.format(
+            self.handler_function, self.queue_name, self.routing_key)
+
 
 def declare(queue_name, routing_key, arguments=None, principal=None):
     return lambda handler_function: Handler(
@@ -41,7 +44,10 @@ def declare(queue_name, routing_key, arguments=None, principal=None):
 handle = declare
 
 
-class ErrorHandlingHandler(object):
+@zope.interface.implementer(
+    gocept.amqprun.interfaces.IHandler,
+    gocept.amqprun.interfaces.IResponse)
+class ErrorHandlingHandler:
 
     queue_name = NotImplemented
     routing_key = NotImplemented
@@ -54,10 +60,6 @@ class ErrorHandlingHandler(object):
     # Exception(s) which are *not* treated in a special way.
     error_reraise = None
 
-    zope.interface.implements(
-        gocept.amqprun.interfaces.IHandler,
-        gocept.amqprun.interfaces.IResponse)
-
     def __init__(self, message=None):
         self.message = message
         self.responses = []
@@ -72,12 +74,14 @@ class ErrorHandlingHandler(object):
         return handler
 
     def handle(self):
+        # The only possible Exception raised from this method is
+        # RetryException. Other Exceptions will be handled and reported by an
+        # deliverable error message.
         try:
             self._decode_message()
             self.run()
-        except self.error_reraise:
-            raise
-        except Exception:
+        except Exception as e:
+            self._handle_retry(e)
             # Solution for https://bitbucket.org/gocept/gocept.amqprun/issue/4:
             # Abort the transaction which caused the error to prevent it from
             # writing any data e. g. to a relational database when committing
@@ -103,25 +107,43 @@ class ErrorHandlingHandler(object):
         raise NotImplementedError()
 
     def exception(self):
+        try:
+            exc = sys.exc_info()[1]
+            self._handle_retry(exc)
+        finally:
+            del exc
         return [self._create_error_message()]
 
-    def send(self, content, routing_key):
-        self.responses.append(self._create_message(content, routing_key))
+    def _handle_retry(self, exc):
+        """(Re-)raise only configured exceptions."""
+        if self.error_reraise and isinstance(exc, self.error_reraise):
+            raise gocept.amqprun.interfaces.RetryException(
+                exc).with_traceback(exc.__traceback__)
 
-    def _create_message(self, content, routing_key):
+    def send(self, content, routing_key, content_encoding=None):
+        self.responses.append(
+            self._create_message(
+                content, routing_key, content_encoding=content_encoding))
+
+    def _create_message(self, content, routing_key, content_encoding=None):
+        if content_encoding:
+            headers = {'content_encoding': content_encoding}
+        else:
+            headers = {}
         return gocept.amqprun.message.Message(
-            {}, content, routing_key=routing_key)
+            headers, content, routing_key=routing_key)
 
     def _create_error_message(self):
         content = '%s\n%s' % self._format_traceback()
-        message = self._create_message(content, self.error_routing_key)
+        message = self._create_message(
+            content, self.error_routing_key, content_encoding='utf-8')
         message.reference(self.message)
         return message
 
     def _format_traceback(self):
         class_, exc, tb = sys.exc_info()
         message = str(exc).replace('\x00', '')
-        message = '%s: %s' % (class_.__name__, message)
+        message = f'{class_.__name__}: {message}'
         detail = ''.join(
             traceback.format_exception(class_, exc, tb)).replace(
                 '\x00', '')
