@@ -1,6 +1,3 @@
-# Copyright (c) 2010-2012 gocept gmbh & co. kg
-# See also LICENSE.txt
-
 import gocept.amqprun.handler
 import gocept.amqprun.interfaces
 import zope.interface
@@ -28,9 +25,8 @@ handler_error = gocept.amqprun.handler.Handler(
     'test.queue.error', 'test.error', handle_message_and_error)
 
 
-class Response(object):
-
-    zope.interface.implements(gocept.amqprun.interfaces.IResponse)
+@zope.interface.implementer(gocept.amqprun.interfaces.IResponse)
+class Response:
 
     responses = []
     _exception = False
@@ -38,7 +34,7 @@ class Response(object):
     def exception(self):
         self._exception = True
         return [gocept.amqprun.message.Message(
-            {}, '', routing_key='test.iresponse-error')]
+            {}, b'', routing_key='test.iresponse-error')]
 
 
 def handle_message_iresponse(message):
@@ -49,13 +45,3 @@ def handle_message_iresponse(message):
 
 handler_iresponse = gocept.amqprun.handler.Handler(
     'test.queue.iresponse', 'test.iresponse', handle_message_iresponse)
-
-
-@gocept.amqprun.handler.declare(
-    'test.queue.counterbelowzero', 'test.counterbelowzero')
-def handler_CounterBelowZero(message):
-    """Handler provoking `CounterBelowZero` after the message was handled."""
-    # This should never be done by a handler as it breaks reference counting
-    # on the channel.
-    message._channel.release()
-    return []
