@@ -1,7 +1,6 @@
-# Copyright (c) 2010-2012 gocept gmbh & co. kg
-# See also LICENSE.txt
-
-import mock
+from unittest import mock
+import amqp.channel
+import gocept.amqprun.message
 import threading
 import time
 import transaction
@@ -14,14 +13,8 @@ import zope.interface.verify
 class DataManagerTest(unittest.TestCase):
 
     def setUp(self):
-        import gocept.amqprun.interfaces
         zope.component.testing.setUp()
-        self.channel_manager = mock.Mock(
-            spec=list(gocept.amqprun.interfaces.IChannelManager))
-        zope.component.provideAdapter(
-            mock.Mock(return_value=self.channel_manager),
-            adapts=(mock.Mock,),
-            provides=gocept.amqprun.interfaces.IChannelManager)
+        self.channel_manager = mock.Mock(spec=amqp.channel.Channel)
         self.connection = mock.Mock()
         self.connection.lock = threading.Lock()
         self.channel = mock.Mock()
@@ -34,7 +27,7 @@ class DataManagerTest(unittest.TestCase):
     def get_message(self):
         import gocept.amqprun.message
         return gocept.amqprun.message.Message(
-            {}, '', 'mytag', channel=self.channel)
+            {}, b'', 'mytag', channel=self.channel)
 
     def get_dm(self):
         import gocept.amqprun.session
@@ -47,19 +40,6 @@ class DataManagerTest(unittest.TestCase):
         zope.interface.verify.verifyObject(
             transaction.interfaces.IDataManager, self.get_dm())
 
-    def test_init_should_acquire_and_release_connection_lock(self):
-        # since getting the current channel from the Server must be locked
-        # against Server.switch_channel
-        def assert_locked():
-            self.assertFalse(self.connection.lock.acquire(False))
-        self.channel_manager.acquire = assert_locked
-        self.get_dm()
-
-    def test_tpc_begin_should_acquire_connection_lock(self):
-        dm = self.get_dm()
-        dm.tpc_begin(None)
-        self.assertFalse(self.connection.lock.acquire(False))
-
     def test_tpc_begin_should_call_tx_select(self):
         dm = self.get_dm()
         dm.tpc_begin(None)
@@ -69,19 +49,13 @@ class DataManagerTest(unittest.TestCase):
         dm = self.get_dm()
         dm.commit(None)
         self.assertTrue(self.channel.basic_ack.called)
-        self.assertEquals(('mytag',), self.channel.basic_ack.call_args[0])
+        self.assertEqual(('mytag',), self.channel.basic_ack.call_args[0])
 
     def test_tpc_vote_should_commit_transaction(self):
         dm = self.get_dm()
         dm.tpc_vote(None)
         self.assertTrue(self.channel.tx_commit.called)
 
-    def test_tpc_finish_should_release_connection_lock(self):
-        dm = self.get_dm()
-        self.connection.lock.acquire()
-        dm.tpc_finish(None)
-        self.assertTrue(self.connection.lock.acquire(False))
-
     def test_tpc_abort_should_release_connection_lock(self):
         dm = self.get_dm()
         dm.tpc_begin(None)
@@ -137,7 +111,9 @@ class DataManagerTest(unittest.TestCase):
         dm.commit(None)
         self.assertEqual(2, self.channel.basic_publish.call_count)
         self.channel.basic_publish.assert_called_with(
-            m2.exchange, m2.routing_key, m2.body, m2.header)
+            m2.as_amqp_message(),
+            exchange=m2.exchange,
+            routing_key=m2.routing_key)
 
     def test_messages_should_be_sent_with_correlation_id(self):
         dm = self.get_dm()
@@ -164,21 +140,22 @@ class DataManagerTest(unittest.TestCase):
         msg = self.get_message()
         self.session.send(msg)
         dm.commit(None)
-        ((_, _, _, header), _) = self.channel.basic_publish.call_args
-        self.assertIn('references', header.headers)
-        self.assertEqual('message id', header.headers['references'])
+        message = self.channel.basic_publish.call_args[0][0]
+        self.assertIn('references', message.headers)
+        self.assertEqual('message id', message.headers['references'])
 
     def test_references_header_contains_parents_references(self):
         dm = self.get_dm()
         dm.session.received_message.header.message_id = 'message id'
-        dm.session.received_message.header.headers = {
+        dm.session.received_message.header.application_headers = {
             'references': 'parent id'}
         msg = self.get_message()
         self.session.send(msg)
         dm.commit(None)
-        ((_, _, _, header), _) = self.channel.basic_publish.call_args
-        self.assertIn('references', header.headers)
-        self.assertEqual('parent id\nmessage id', header.headers['references'])
+        message = self.channel.basic_publish.call_args[0][0]
+        self.assertIn('references', message.headers)
+        self.assertEqual(
+            'parent id\nmessage id', message.headers['references'])
 
     def test_no_references_when_parent_has_no_reference_header(self):
         dm = self.get_dm()
@@ -187,30 +164,31 @@ class DataManagerTest(unittest.TestCase):
         msg = self.get_message()
         self.session.send(msg)
         dm.commit(None)
-        ((_, _, _, header), _) = self.channel.basic_publish.call_args
-        self.assertIn('references', header.headers)
-        self.assertEqual('message id', header.headers['references'])
+        message = self.channel.basic_publish.call_args[0][0]
+        self.assertIn('references', message.headers)
+        self.assertEqual('message id', message.headers['references'])
 
     def test_existing_references_header_should_not_be_overwritten(self):
         dm = self.get_dm()
         dm.session.received_message.header.message_id = 'message id'
-        msg = mock.Mock()
-        msg.header.headers = {'references': 'custom id'}
+        msg = gocept.amqprun.message.Message(
+            {'references': 'custom id'},
+            b'body'
+        )
         self.session.send(msg)
         dm.commit(None)
-        ((_, _, _, header), _) = self.channel.basic_publish.call_args
-        self.assertEqual('custom id', header.headers['references'])
+        message = self.channel.basic_publish.call_args[0][0]
+        self.assertEqual('custom id', message.headers['references'])
 
     def test_no_references_should_be_created_when_parent_lacks_message_id(
             self):
         dm = self.get_dm()
         dm.session.received_message.header.message_id = None
-        msg = mock.Mock()
-        msg.header.headers = {}
+        msg = self.get_message()
         self.session.send(msg)
         dm.commit(None)
-        ((_, _, _, header), _) = self.channel.basic_publish.call_args
-        self.assertNotIn('references', header.headers)
+        message = self.channel.basic_publish.call_args[0][0]
+        self.assertNotIn('references', message.headers)
 
     def test_abort_should_discard_queued_messages(self):
         dm = self.get_dm()
@@ -246,11 +224,9 @@ class DataManagerTest(unittest.TestCase):
 class SessionTest(unittest.TestCase):
 
     def setUp(self):
-        self.patcher = mock.patch('gocept.amqprun.session.AMQPDataManager')
-        self.patcher.__enter__()
-
-    def tearDown(self):
-        self.patcher.__exit__()
+        patcher = mock.patch('gocept.amqprun.session.AMQPDataManager')
+        patcher.start()
+        self.addCleanup(patcher.stop)
 
     def create_session(self, message=None):
         from gocept.amqprun.session import Session
